#!/usr/bin/python
# tooltable.py - tool offset table widget. Used by pymini.py.
#
# (c) 2014-2016 Copyright Eckler Software
#
# Author: David Suffield, dsuffiel@ecklersoft.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of version 2 of the GNU General Public License as published by
# the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
# Upstream patches are welcome. Any patches submitted to the author must be
# unencumbered (ie: no Copyright or License).
#

import os, sys, logging
try:
    import tkinter
except ImportError:
    import Tkinter as tkinter

class ToolTable(tkinter.Frame):
    """ToolTable widget"""
    def __init__(self, parent=None, tool_file=""):
        tkinter.Frame.__init__(self, parent)
        self.tool_file = tool_file
        self.tool_sel = 1
        self.parent = parent
        self.result = None

        self.table = self.table_read()

        grow = 0
        self.toolnum_label = tkinter.Label(self, text="toolnum")
        self.toolnum_label.grid(row=0, column=0, sticky='ew')
        self.toolnum_val = tkinter.StringVar()
        self.toolnum_entry = tkinter.Label(self, textvariable=self.toolnum_val, relief=tkinter.SUNKEN)
        self.toolnum_entry.grid(row=0, column=1, columnspan=3, sticky='ew')

        grow += 1
        self.tlength_label = tkinter.Label(self, text="length")
        self.tlength_label.grid(row=grow, column=0, sticky='ew')
        self.tlength_val = tkinter.StringVar()
        self.tlength_entry = tkinter.Entry(self, textvariable=self.tlength_val)
        self.tlength_entry.grid(row=grow, column=1, columnspan=3, sticky='ew')

        grow += 1
        self.tdiameter_label = tkinter.Label(self, text="diameter")
        self.tdiameter_label.grid(row=grow, column=0, sticky='ew')
        self.tdiameter_val = tkinter.StringVar()
        self.tdiameter_entry = tkinter.Entry(self, textvariable=self.tdiameter_val)
        self.tdiameter_entry.grid(row=grow, column=1, columnspan=3, sticky='ew')

        grow += 1
        self.tcomment_label = tkinter.Label(self, text="comment")
        self.tcomment_label.grid(row=grow, column=0, sticky='ew')
        self.tcomment_val = tkinter.StringVar()
        self.tcomment_entry = tkinter.Entry(self, textvariable=self.tcomment_val)
        self.tcomment_entry.grid(row=grow, column=1, columnspan=3, sticky='ew')

        # Update table values.
        self.update()

        grow += 1
        self.back_button = tkinter.Button(self, text="Back", command=self.back)
        self.back_button.grid(row=grow, column=0)
        self.next_button = tkinter.Button(self, text="Next", command=self.next)
        self.next_button.grid(row=grow, column=1)
        self.ok_button = tkinter.Button(self, text="OK", command=self.ok)
        self.ok_button.grid(row=grow, column=2)
        self.cancel_button = tkinter.Button(self, text="Cancel", command=self.cancel)
        self.cancel_button.grid(row=grow, column=3)

    def save(self):
        """Write selected dialog entry into dictionary."""
        self.table[self.tool_sel]['length'] = float(self.tlength_val.get())
        self.table[self.tool_sel]['diameter'] = float(self.tdiameter_val.get())
        self.table[self.tool_sel]['comment'] = self.tcomment_val.get()

    def update(self):
        """Write dictionary entry into dialog."""
        self.toolnum_val.set("%d" % (self.tool_sel))
        self.tlength_val.set("%g" % (self.table[self.tool_sel]['length']))
        self.tdiameter_val.set("%g" % (self.table[self.tool_sel]['diameter']))
        self.tcomment_val.set("%s" % (self.table[self.tool_sel]['comment']))

    def back(self):
        self.save()
        if (self.tool_sel > 1):
            self.tool_sel -= 1
        self.update()

    def next(self):
        self.save()
        if (self.tool_sel < len(self.table)):
            self.tool_sel += 1
        self.update()

    def ok(self, event=None):
        try:        
            self.save()
        except Exception as err:
            logging.error("Unable save tool offsets: %s" % (err))
            return  # leave dialog open

        self.result = self.table

        # Close old stepper.tbl file.
        self.parent.dog.close()
        try:
            self.table_write(self.table)
            logging.info("Saved tool offsets: %s" % (self.tool_file))            
            # Open new stepper.tbl file.
            self.parent.dog.open(self.parent.homedir, self.parent.inifile)           
        except Exception as err:
            logging.error("Unable save tool offsets: %s" % (err))
            return # leave dialog open
        self.cancel()

    def table_read(self):
        tbl = {}
        self.default_write()
        if (not os.path.exists(self.tool_file)):
            logging.error("Unable to load tool table file: %s" % (self.tool_file))
            return tbl
        with open(self.tool_file, 'r') as f:
            line = f.readline()
            while (line != ''):
                if (line[0] == '#'):
                    line = f.readline()
                    continue
                ln = line.split(None, 4)
                if (len(ln) != 5):
                    line = f.readline()
                    continue
                tool = {}
                tool['length'] = float(ln[2])
                tool['diameter'] = float(ln[3])
                tool['comment'] = ln[4].strip()
                tbl[int(ln[0])] = tool
                line = f.readline()
        return tbl

    def table_write(self, tbl):
        # Backup the old table.
        if (os.path.exists(self.tool_file)):
            backup = "%s.bak" % (self.tool_file)
            if (os.path.exists(backup)):
                os.remove(backup)
            os.rename(self.tool_file, backup)

        with open(self.tool_file, 'w') as f:
            f.write("# toolnum  pocket  length  diameter  comment")
            f.write("\n")
            for i in range(1, len(tbl)+1):
                tool = tbl.get(i)
                f.write("%d   %d   %g   %g   %s\n" % (i, i, tool['length'], tool['diameter'], tool['comment']))
            f.write("\n")

    def cancel(self, event=None):
        """Hide this widget."""
        self.grid_remove()

    def default_write(self):
        """Write default stepper.tbl file if it does't exist."""
        if (os.path.exists(self.tool_file)):
            return
        # Create a default tool table with zero offsets.
        tbl = {}
        for i in range(1, 11):
            tool = {}
            tool['length'] = 0.0
            tool['diameter'] = 0.0
            tool['comment'] = "empty"
            tbl[i] = tool
        self.table_write(tbl)
        
    def button_state(self, s):
        self.ok_button.config(state=s)

    def table_reload(self):
        """Reload dialog from file."""
        self.table = self.table_read()
        self.update()

if __name__ == "__main__":
    logging.basicConfig(stream=sys.stdout, level=logging.DEBUG, format='%(message)s')

    root = tkinter.Tk()
    tool_file = "%s/.pymini/stepper.tbl" % (os.path.expanduser("~"))
    d = ToolTable(root, tool_file)

